/*  File: analyser-jets.cc
 *  Project: Deductor
 *  Created by Zoltán Nagy on 9/8/13.
 *  Modified by D. Soper on 9 July 2017
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/*   local headers  */
#include "analyzer-jets.h"



void UserJets::initfunc()
{
  /*   Create histogram to calculate the one jet cross section   */
  auto bins = spacing<hist1d>::manual({290.0,
    310.0,330.0,350.0,370.0,390.0,
    410.0,430.0,450.0,470.0,490.0,
    510.0,530.0,550.0,570.0,590.0,
    610.0,630.0,650.0,670.0,690.0,
    710.0,730.0,750.0,770.0,790.0,
    810.0,830.0,850.0,870.0,890.0,
    910.0,930.0,950.0,970.0,990.0,1010.0,1030.0,
    1050.0,1150.0,1250.0,1350.0,1450.0,1550.0,1650.0,1750.0,1850.0,1950.0,
    2050.0,2150.0,2250.0,2350.0,2450.0,2550.0,2650.0,2750.0,2850.0,2950.0,
    3050.0,3150.0,3250.0,3350.0,3450.0,3550.0,3650.0,3750.0,3850.0,3950.0,
    4050.0,4150.0,4250.0,4350.0,4450.0,4550.0,4650.0,4750.0,4850.0,4950.0,5050.0});
  /* This was defined as a 4 dimensional array of histograms  */
  HistArr::phys(1, "One jet inclusive cross section NLO pdf", bins);
  
  /*   PT changes from shower    */
  unsigned int nrbins = 80;
  double rmin = 0.6, rmax = 1.4;
  auto rbins = spacing<hist1d>::linear(nrbins, rmin, rmax);
  Hist1D::phys(3, "PTjet/PTborn $0.8 < r < 1.2$", rbins);
  //auto hardbins = spacing<hist1d>::manual({0.0,2800.0,3300.0,5000.0});
  Hist0D::phys(4, "Born cross section for PT near 3 TeV, NLO pdf");
  
  cout << "UserJets is using " << jetDef.description() << endl;
  
  ptcodecount = 0;
  usercount = 0;
}


void UserJets::userfunc(const shower_history<hhc>& history)
{
  constexpr double tonb = 389379.338;
  usercount++;
  double pTborn = 0.0,  y1 = 0.0, y2 = 0.0;

  //   One jet inclusive cross section before showering
  //  --------------------------------------------------
  int hardcount = 0;
  for(auto& hard : history.hard())
  {
    hardcount++;
    if(hardcount < 2) // Count the hard event once only.
    {
      /*    hard events   */
      auto& q = hard.state;
      std::array<double, 4u> w1{{0.0, 0.0, 0.0, 0.0}};
      double hweight = hard.weights.unitary.main*color_weight(hard)*tonb;
      w1[3]  = hweight;

      pTborn = q[1].momentum.perp();
      y1 = abs(q[1].momentum.pseudo_rapidity());
      y2 = abs(q[2].momentum.pseudo_rapidity());
      
      /*   Fill component [3] of main histogram.
       *   Only count jets that have |y| < 2.0
       *   The two jets are back-to-back, since we have only two
       *   partons in the final state.
       */
      if(y1 < 2.0) HistArr::physfill(1, w1, dirac{}, pTborn);
      if(y2 < 2.0) HistArr::physfill(1, w1, dirac{}, pTborn);
      
      /* Also, we fill the 0 dimensional historgram for the hard state
       * used when computing the distribution of pT fractions
       */
      if(y1 < 2.0 && y2 < 2.0 &&
         pTborn > 2800.0 && pTborn < 3300.0)
      {
        ptcodecount++;
        Hist0D::physfill(4, hweight);
      }
    } //End if(hardcount < 2)
  } //End for(auto& hard : history.hard())

  //   One jet inclusive cross section after showering
  //  -------------------------------------------------
  for(auto& showered : history)
  {
    /*   showered events   */
    auto& p = showered.state;
    
    std::array<double, 4u> w1;

    w1[3] = 0.0;
    w1[2]  = showered.weights.unitary.main*color_weight(showered)*tonb;
    w1[1]  = w1[2]*showered.weights.threshold.main*showered.weights.threshold.reg;
    w1[0]  = w1[1]*showered.weights.threshold.delta;
    
    double w3 = w1[2];
    
    /*   Converting deductor event to FastJet format   */
    make_pseudojets(theparticles, p);
    
    /*    Run fastjet algorithm   */
    fj::ClusterSequence clustSeq(theparticles, jetDef);
    
    /*    Extract inclusive jets sorted by pT (above pTjetmin)   */
    double pTjetmin = 200.0_GeV;
    incJets = clustSeq.inclusive_jets(pTjetmin);
    
    /*    Fill histograms, only count jets that have |y| < 2.0
     *    Component [3] of histogram 1 was filled with hard cross section.
     */
    for(auto& pj : incJets)
    {
      if(abs(pj.pseudorapidity()) < 2.0)
      {
        double pT = pj.perp();
        HistArr::physfill(1, w1, dirac{}, pT);
      }
      if(y1 < 2.0 && y2 < 2.0 &&
         pTborn > 2800.0 && pTborn < 3300.0 &&
         abs(pj.pseudorapidity()) < 2.0)
      {
        double pTratio = pj.perp()/pTborn;
        Hist1D::physfill(3, w3, dirac{}, pTratio);
      }
    }
  }
}  // End void UserJets::userfunc(...)

