// Adapted from main11.cc.
// main11.cc is a part of the PYTHIA event generator.
// It illustrates how Les Houches Event File input can be used in Pythia8.
// Copyright (C) 2015 Torbjorn Sjostrand.
// PYTHIA is licenced under the GNU GPL version 2, see COPYING for details.
// Please respect the MCnet Guidelines, see GUIDELINES for details.

// This program uses input from Deductor, with information transfer
// in the style of a Les Houches Event File.
// It also uses FastJet.
// D.E. Soper, 12 September 2017

// To use this, first execute the system command
//
//    mkfifo jetevents.lhe
//
// We examine the inclusive jet cross section and also
// look at the transverse-min sum pT distribution.
// Cf. Eur. Phys. J. C (2014) 74:2965

#include "Pythia8/Pythia.h"
#include "fastjet/PseudoJet.hh"
#include "fastjet/ClusterSequence.hh"
#include <iostream>
#include <fstream>
#include <string>
using namespace Pythia8;
int main() {
  
  // Fastjet analysis - select algorithm and parameters
  double Rparam = 0.6;
  fastjet::Strategy               strategy = fastjet::Best;
  fastjet::RecombinationScheme    recombScheme = fastjet::E_scheme;
  fastjet::JetDefinition         *jetDef = NULL;
  jetDef = new fastjet::JetDefinition(fastjet::antikt_algorithm, Rparam,
                                      recombScheme, strategy);
  // Fastjet input
  std::vector <fastjet::PseudoJet> fjInputs;
  
  /* Define pTcut */
  double pTcut = 20.0;
  
  cout << "PythiaGaplhe ready to open file." << endl;

  ifstream lhe;
  lhe.open("jetevents.lhe");
  string word, line;
  int hadronA     = 123456789;
  int hadronB     = 123456789;
  double EA       = 123456789.0;
  double EB       = 123456789.0;
  int npartons    = 123456789;
  int processcode = 123456789;
  double weight   = 123456789.0;
  bool ducteventOK= false;
  int flavor      = 123456789;
  int statuscode  = 123456789;
  int mother1     = 123456789;
  int mother2     = 123456789;
  int colorL      = 123456789;
  int colorR      = 123456789;
  double Px       = 123456789.0;
  double Py       = 123456789.0;
  double Pz       = 123456789.0;
  double E        = 123456789.0;
  double mass     = 123456789.0;

  // We will get NDeductorEvents from Deductor.
  // Note that when opts.allow_color_branching = true in deductor,
  // each Deductor event can have several branches, resulting in
  // several "events" being sent to Pythia.
  int NDeductorEvents = 123456789;
  
  // Reading of initial part of jetevents.lhe
  while (lhe >> word)
  {
    if (word == "<init>") // Find the "init" section.
    {
      getline(lhe,line);  // Go to the next line.
      break;
    }
  }
  lhe >> hadronA >> hadronB >> EA >> EB; // Read beams information.
  while (lhe >> word)  // Find end of the "init" section.
  {
    if (word == "</init>")
    {
      getline(lhe,line); // Go to the next line.
      break;
    }
  }
  
  // Now we are ready to read events.

  // Generator.
  // Follow "Hadron-Level Standalone"
  Pythia pythia;
  Event& event = pythia.event;                 // Convenient shorthand.
  pythia.readString("ProcessLevel:all = off"); // The trick!
  pythia.settings.parm("Beams:idA", hadronA);
  pythia.settings.parm("Beams:idB", hadronB);
  pythia.settings.parm("Beams:eCM", EA + EB);
  pythia.readString("Next:numberCount = 100000");  // Show iEvent every numberCount events.
  pythia.readString("TimeShower:MEcorrections = off");
  
  pythia.init();                               // Initialization.
  
  // Book histograms.
  Hist transminPTdist("Transverse-min PT/Delta R",100,0.0,10.0);
  
  Hist y1A("y1_A",4,50.0,90.0);
  Hist y1B("y1_B",4,90.0,150.0);
  Hist y1C("y1_C",6,150.0,340.0);
  Hist y1D("y1_D",4,340.0,500.0);
  Hist y2A("y2_A",4,50.0,90.0);
  Hist y2B("y2_B",4,90.0,150.0);
  Hist y2C("y2_C",6,150.0,340.0);
  Hist y2D("y2_D",4,340.0,500.0);
  Hist y3A("y3_A",4,50.0,90.0);
  Hist y3B("y3_B",4,90.0,150.0);
  Hist y3C("y3_C",6,150.0,340.0);
  Hist y3D("y3_D",4,340.0,500.0);
  Hist y4A("y4_A",4,50.0,90.0);
  Hist y4B("y4_B",4,90.0,150.0);
  Hist y4C("y4_C",6,150.0,340.0);
  Hist y4D("y4_D",4,340.0,500.0);
  Hist y5A("y5_A",4,50.0,90.0);
  Hist y5B("y5_B",4,90.0,150.0);
  Hist y5C("y5_C",6,150.0,340.0);
  Hist y5D("y5_D",4,340.0,500.0);
  Hist y101A("y101_A",4,50.0,90.0);
  Hist y101B("y101_B",4,90.0,150.0);
  Hist y101C("y101_C",6,150.0,340.0);
  Hist y101D("y101_D",4,340.0,500.0);
  Hist y102A("y102_A",4,50.0,90.0);
  Hist y102B("y102_B",4,90.0,150.0);
  Hist y102C("y102_C",6,150.0,340.0);
  Hist y102D("y102_D",4,340.0,500.0);
  Hist y103A("y103_A",4,50.0,90.0);
  Hist y103B("y103_B",4,90.0,150.0);
  Hist y103C("y103_C",6,150.0,340.0);
  Hist y103D("y103_D",4,340.0,500.0);
  Hist y104A("y104_A",4,50.0,90.0);
  Hist y104B("y104_B",4,90.0,150.0);
  Hist y104C("y104_C",6,150.0,340.0);
  Hist y104D("y104_D",4,340.0,500.0);
  Hist y105A("y105_A",4,50.0,90.0);
  Hist y105B("y105_B",4,90.0,150.0);
  Hist y105C("y105_C",6,150.0,340.0);
  Hist y105D("y105_D",4,340.0,500.0);
  //For results at intermediate stages:
  Hist yy1A("yy1_A",4,50.0,90.0);
  Hist yy1B("yy1_B",4,90.0,150.0);
  Hist yy1C("yy1_C",6,150.0,340.0);
  Hist yy1D("yy1_D",4,340.0,500.0);
  Hist yy2A("yy2_A",4,50.0,90.0);
  Hist yy2B("yy2_B",4,90.0,150.0);
  Hist yy2C("yy2_C",6,150.0,340.0);
  Hist yy2D("yy2_D",4,340.0,500.0);
  Hist yy3A("yy3_A",4,50.0,90.0);
  Hist yy3B("yy3_B",4,90.0,150.0);
  Hist yy3C("yy3_C",6,150.0,340.0);
  Hist yy3D("yy3_D",4,340.0,500.0);
  Hist yy4A("yy4_A",4,50.0,90.0);
  Hist yy4B("yy4_B",4,90.0,150.0);
  Hist yy4C("yy4_C",6,150.0,340.0);
  Hist yy4D("yy4_D",4,340.0,500.0);
  Hist yy5A("yy5_A",4,50.0,90.0);
  Hist yy5B("yy5_B",4,90.0,150.0);
  Hist yy5C("yy5_C",6,150.0,340.0);
  Hist yy5D("yy5_D",4,340.0,500.0);


  double weightsum = 0.0;
  int iEvent = 0;
  // Read events.
  while (lhe >> word)
  {
    if (word == "</LesHouchesEvents>")
    {
      cout << "Tag </LesHouchesEvents> found at event " << iEvent << endl;
      lhe >> word;
      cout << word;
      lhe >> NDeductorEvents;
      cout << " " << NDeductorEvents << " " ;
      lhe >> word;
      cout << word << " ";
      lhe >> word;
      cout << word << endl;
      break;
    }
    
    ducteventOK = true;
    if (word == "<event>")
    {
      event.reset();     // This resets the event record & creates line 0.
      getline(lhe,line); // Go to the next line.
      iEvent++;          // Increment the event index.
      lhe >> npartons >> processcode >> weight;
      if(iEvent < 6)
      {
        cout << scientific;
        cout << "event " << iEvent << ": npartons: " << npartons << " weight: " << weight << endl;
      }
      if(!(abs(weight) < 1.0e10))
      {
        cout << "event " << iEvent << ": npartons: " << npartons << " weight: " << weight << endl;
        weight = 0.0;
        ducteventOK = false;
      }
      getline(lhe,line);  // Go to the next line.
      
      double Pxsum    = 0.0;
      double Pysum    = 0.0;
      double Pzsum    = 0.0;
      double Esum     = 0.0;
      for (int n = 1; n <= npartons; n++)
      {
        lhe >> flavor >> statuscode >> mother1 >> mother2  >> colorL >> colorR
        >> Px >> Py >> Pz >> E >> mass;
        getline(lhe,line);  // Go to the next line.
        
        Pxsum += Px;
        Pysum += Py;
        Pzsum += Pz;
        Esum  += E;
 
        if (statuscode ==  1)
        {
          statuscode =  23;
          if (flavor == 0) flavor = 21;
          
          if (iEvent < 6 || !ducteventOK)
          {
            cout << std::showpoint;
            cout << "f: " <<  setw(2) << flavor << " status: " << setw(2) << statuscode
            << " colorL: " << setw(4) << colorL << " colorR: " << setw(4) << colorR
            << " Px: " << setw(10) << Px << " Py: " << setw(10) << Py << " Pz: " << setw(11) << Pz
            << " E: " << setw(10) << E << " m2: " << setw(10) << mass*mass
            << " m2calc: " << setw(10) << E*E - Px*Px - Py*Py - Pz*Pz  << endl;
          }
          
          // Add this parton to the event record.
          event.append(flavor, statuscode, colorL, colorR, Px, Py, Pz, E, mass);
        }
      }
      if (iEvent < 6)
      {
        cout << "Pxsum = " << Pxsum << " Pysum = " << Pysum
        << " Pzsum = " << Pzsum << " Esum = " << Esum << endl << endl;
      }
      lhe >> word;
      if (word != "</event>")  cout << "lhe format error.";
    } // Done with reading event: if (word == "<event>")

    // Hadronize the event.
    if(ducteventOK)
    {
      bool pythiaeventOK = pythia.next();
      if (!pythiaeventOK)
      {
        cout << "Pythia event " << iEvent << " has a problem." << endl;
        // pythia.event.list();
      }
      if(pythiaeventOK)
      {
        // Reset Fastjet input
        fjInputs.resize(0);
        // Loop over event record to decide what to pass to FastJet
        for (int i = 0; i < pythia.event.size(); ++i)
        {
          // Final state only
          if (!pythia.event[i].isFinal())        continue;
          
          // No neutrinos
          if (pythia.event[i].idAbs() == 12 || pythia.event[i].idAbs() == 14 ||
              pythia.event[i].idAbs() == 16)     continue;
          
          // Only |eta| < 10.0  ** This was changed **
          if (fabs(pythia.event[i].eta()) > 10.0) continue;
          
          // Store as input to Fastjet
          fjInputs.push_back( fastjet::PseudoJet(pythia.event[i].px(),
                                                 pythia.event[i].py(),
                                                 pythia.event[i].pz(),
                                                 pythia.event[i].e() ) );
        }
        if (fjInputs.size() == 0)
        {
          cout << "Error: event with no final state particles" << endl;
          continue;
        }
        // Run Fastjet algorithm
        fastjet::ClusterSequence clustSeq(fjInputs, *jetDef);
        
        // For the first event, print the FastJet details
        if (iEvent < 2)
        {
          cout << "Ran " << jetDef->description() << endl;
          cout << "Strategy adopted by FastJet was "
          << clustSeq.strategy_string() << endl << endl;
        }
        // Extract inclusive jets sorted by pT (note minimum pT)
        vector <fastjet::PseudoJet> incJets0 = clustSeq.inclusive_jets(pTcut);
        vector <fastjet::PseudoJet> incJets  = sorted_by_pt(incJets0);
        // Find two leading PT jets in allowed rapidity range.
        fastjet::PseudoJet jet[2];
        unsigned int nj = 0;
        for(unsigned int n = 0; n < incJets.size() && nj < 2; n++)
          if(std::abs(incJets[n].rapidity()) < 4.4) jet[nj++] = incJets[n];
        
        bool jetsOK = true;
        if(nj < 2 ) jetsOK = false;
        if(jetsOK)
        {
          double Ymin = jet[0].rapidity(), Ymax = jet[1].rapidity();
          double DeltaY = std::abs(Ymax - Ymin);
          if(Ymin > Ymax) std::swap(Ymin, Ymax);
          double pTaverage = 0.5*(jet[0].perp() + jet[1].perp());
          
          double tiny = 1.0e-10;
          double wgap = weight; // wg is weight if nothing in gap, otherwise 0.
          
          for(unsigned int i = 0; i < incJets.size(); i++)
            if(incJets[i].pt() > pTcut &&
               incJets[i].rapidity() < Ymax - tiny &&
               incJets[i].rapidity() > Ymin + tiny )
            {
              wgap = 0.0;
              break;
            }

          if(1.0 < DeltaY && DeltaY < 2.0)
          {
            y1A.fill(pTaverage,weight);
            y1B.fill(pTaverage,weight);
            y1C.fill(pTaverage,weight);
            y1D.fill(pTaverage,weight);
            y101A.fill(pTaverage,wgap);
            y101B.fill(pTaverage,wgap);
            y101C.fill(pTaverage,wgap);
            y101D.fill(pTaverage,wgap);
          }
          if (2.0 < DeltaY && DeltaY < 3.0)
          {
            y2A.fill(pTaverage,weight);
            y2B.fill(pTaverage,weight);
            y2C.fill(pTaverage,weight);
            y2D.fill(pTaverage,weight);
            y102A.fill(pTaverage,wgap);
            y102B.fill(pTaverage,wgap);
            y102C.fill(pTaverage,wgap);
            y102D.fill(pTaverage,wgap);
          }
          
          if(3.0 < DeltaY && DeltaY < 4.0)
          {
            y3A.fill(pTaverage,weight);
            y3B.fill(pTaverage,weight);
            y3C.fill(pTaverage,weight);
            y3D.fill(pTaverage,weight);
            y103A.fill(pTaverage,wgap);
            y103B.fill(pTaverage,wgap);
            y103C.fill(pTaverage,wgap);
            y103D.fill(pTaverage,wgap);
          }
          
          if (4.0 < DeltaY && DeltaY < 5.0)
          {
            y4A.fill(pTaverage,weight);
            y4B.fill(pTaverage,weight);
            y4C.fill(pTaverage,weight);
            y4D.fill(pTaverage,weight);
            y104A.fill(pTaverage,wgap);
            y104B.fill(pTaverage,wgap);
            y104C.fill(pTaverage,wgap);
            y104D.fill(pTaverage,wgap);
          }
          
          if(5.0 < DeltaY && DeltaY < 6.0)
          {
            y5A.fill(pTaverage,weight);
            y5B.fill(pTaverage,weight);
            y5C.fill(pTaverage,weight);
            y5D.fill(pTaverage,weight);
            y105A.fill(pTaverage,wgap);
            y105B.fill(pTaverage,wgap);
            y105C.fill(pTaverage,wgap);
            y105D.fill(pTaverage,wgap);
          }

        } // End if(jetsOK)
        
        // Transverse-min PT distribution
        if (incJets.size()>0)
        {
          double px = incJets[0].py();
          double py = - incJets[0].px();
          double norm = sqrt(px*px + py*py);
          double nx = px/norm;
          double ny = py/norm;
          if (norm > 300.0)
          {
            double pTsum1 = 0.0;
            double pTsum2 = 0.0;
            double pTsum = 0.0;
            for (int i = 0; i < pythia.event.size(); ++i)
            {
              if (pythia.event[i].isFinal() && pythia.event[i].isCharged())
              {
                double kx = pythia.event[i].px();
                double ky = pythia.event[i].py();
                double absk = sqrt(kx*kx + ky*ky);
                double costheta = (kx*nx + ky*ny)/absk;
                double eta = pythia.event[i].eta(); // pseudorapidity
                if ( (abs(costheta) > sqrt(3.0)/2.0) && (abs(eta) < 2.5) && (absk > 0.5))
                {
                  if( costheta > 0) pTsum1 += pythia.event[i].pT();
                  else pTsum2 += pythia.event[i].pT();
                }
              }
            } // End loop over Pythia particles.
            // Find pTsum, add to total weight and fill in histogram.
            pTsum = min(pTsum1,pTsum2); // The transverse-min pT sum
            pTsum /= 10.4719755120;     // Divide by solid angle = 10*Pi/3
            weightsum += weight;
            transminPTdist.fill(pTsum, weight);
          } // End if (norm > 300.0)
        } // End analysis of transminPTdist: if (sortedJets.size()>0)
        
      } // End if(pythiaeventOK)
    } //End if(ducteventOK)
    
    // Write intermediate results:
    if (iEvent > 100 && iEvent%1000000 == 0)
    {
      yy1A = y101A/y1A;
      yy1B = y101B/y1B;
      yy1C = y101C/y1C;
      yy1D = y101D/y1D;
      
      yy2A = y102A/y2A;
      yy2B = y102B/y2B;
      yy2C = y102C/y2C;
      yy2D = y102D/y2D;
      
      yy3A = y103A/y3A;
      yy3B = y103B/y3B;
      yy3C = y103C/y3C;
      yy3D = y103D/y3D;
      
      yy4A = y104A/y4A;
      yy4B = y104B/y4B;
      yy4C = y104C/y4C;
      yy4D = y104D/y4D;
      
      yy5A = y105A/y5A;
      yy5B = y105B/y5B;
      yy5C = y105C/y5C;
      yy5D = y105D/y5D;
      
      ofstream tempfile;
      
      tempfile.open ("pythiaductGapY1.txt");
      tempfile << "# Gap ratio.\n";
      tempfile << "# 1 < Delta y < 2 \n";
      tempfile << "# " << iEvent << " events. \n";
      yy1A.table(tempfile);
      yy1B.table(tempfile);
      yy1C.table(tempfile);
      yy1D.table(tempfile);
      tempfile.close();
      
      tempfile.open ("pythiaductGapY2.txt");
      tempfile << "# Gap ratio.\n";
      tempfile << "# 2 < Delta y < 3 \n";
      tempfile << "# " << iEvent << " events. \n";
      yy2A.table(tempfile);
      yy2B.table(tempfile);
      yy2C.table(tempfile);
      yy2D.table(tempfile);
      tempfile.close();
      
      tempfile.open ("pythiaductGapY3.txt");
      tempfile << "# Gap ratio.\n";
      tempfile << "# 3 < Delta y < 4 \n";
      tempfile << "# " << iEvent << " events. \n";
      yy3A.table(tempfile);
      yy3B.table(tempfile);
      yy3C.table(tempfile);
      yy3D.table(tempfile);
      tempfile.close();
      
      tempfile.open ("pythiaductGapY4.txt");
      tempfile << "# Gap ratio.\n";
      tempfile << "# 4 < Delta y < 5 \n";
      tempfile << "# " << iEvent << " events. \n";
      yy4A.table(tempfile);
      yy4B.table(tempfile);
      yy4C.table(tempfile);
      yy4D.table(tempfile);
      tempfile.close();
      
      tempfile.open ("pythiaductGapY5.txt");
      tempfile << "# Gap ratio.\n";
      tempfile << "# 5 < Delta y < 6 \n";
      tempfile << "# " << iEvent << " events. \n";
      yy5A.table(tempfile);
      yy5B.table(tempfile);
      yy5C.table(tempfile);
      yy5D.table(tempfile);
      tempfile.close();
    }
    
    
    
  } // Done with reading events: while (lhe >> word).
  cout << "Done reading " << iEvent << " events sent to Pythia out of "
       << NDeductorEvents << " created by Deductor." << endl;
  lhe.close();
  
  // Give statistics. Print histograms.
  pythia.stat();
  cout << "The total weight is " << weightsum << endl;
  transminPTdist = transminPTdist/weightsum;
  cout << transminPTdist;
  transminPTdist.table();
  
  y1A = y101A/y1A;
  y1B = y101B/y1B;
  y1C = y101C/y1C;
  y1D = y101D/y1D;
  
  y2A = y102A/y2A;
  y2B = y102B/y2B;
  y2C = y102C/y2C;
  y2D = y102D/y2D;

  y3A = y103A/y3A;
  y3B = y103B/y3B;
  y3C = y103C/y3C;
  y3D = y103D/y3D;
  
  y4A = y104A/y4A;
  y4B = y104B/y4B;
  y4C = y104C/y4C;
  y4D = y104D/y4D;
  
  y5A = y105A/y5A;
  y5B = y105B/y5B;
  y5C = y105C/y5C;
  y5D = y105D/y5D;
  
  ofstream myfile;
  
  myfile.open ("pythiaductGapY1.dat");
  myfile << "# Gap ratio.\n";
  myfile << "# 1 < Delta y < 2 \n";
  myfile << "# " << NDeductorEvents << " events. \n";
  y1A.table(myfile);
  y1B.table(myfile);
  y1C.table(myfile);
  y1D.table(myfile);
  myfile.close();
  
  myfile.open ("pythiaductGapY2.dat");
  myfile << "# Gap ratio.\n";
  myfile << "# 2 < Delta y < 3 \n";
  myfile << "# " << NDeductorEvents << " events. \n";
  y2A.table(myfile);
  y2B.table(myfile);
  y2C.table(myfile);
  y2D.table(myfile);
  myfile.close();
  
  myfile.open ("pythiaductGapY3.dat");
  myfile << "# Gap ratio.\n";
  myfile << "# 3 < Delta y < 4 \n";
  myfile << "# " << NDeductorEvents << " events. \n";
  y3A.table(myfile);
  y3B.table(myfile);
  y3C.table(myfile);
  y3D.table(myfile);
  myfile.close();
  
  myfile.open ("pythiaductGapY4.dat");
  myfile << "# Gap ratio.\n";
  myfile << "# 4 < Delta y < 5 \n";
  myfile << "# " << NDeductorEvents << " events. \n";
  y4A.table(myfile);
  y4B.table(myfile);
  y4C.table(myfile);
  y4D.table(myfile);
  myfile.close();
  
  myfile.open ("pythiaductGapY5.dat");
  myfile << "# Gap ratio.\n";
  myfile << "# 5 < Delta y < 6 \n";
  myfile << "# " << NDeductorEvents << " events. \n";
  y5A.table(myfile);
  y5B.table(myfile);
  y5C.table(myfile);
  y5D.table(myfile);
  myfile.close();
  
  // Done.
  return 0;
}
