/*  File: analyser-AtlasGap.cc
 *  Project: Deductor
 *  Created by D. Soper on 2 December 2014
 *  Modified by D. Soper on 3 November 2017
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/*   std includes   */

/*   project headers  */

/*   local headers  */
#include "analyzer-gap.h"

/*   other includes   */

void UserGap::initfunc()
{
  /*  Create histograms to calculate the fraction of two jet events with a gap
   *  as a function of the PTcut for the gap.
   *  We demand that there are two jets with PT > PTjet GeV
   *  one with Y < - Ygap and one with Y > + Ygap.
   *  Then there can be no jet in the gap -Ygap < Y < +Ygap
   *  with PT > PTcut 
   */
  auto bins = spacing<hist1d>::manual({50,60,70,80,90,105,120,135,150,180,
       210,240,270,300,340,380,420,460,500});
  
  Hist1D::phys(1, "Cross section, 1 < Delta y < 2", bins);
  Hist1D::phys(2, "Cross section, 2 < Delta y < 3", bins);
  Hist1D::phys(3, "Cross section, 3 < Delta y < 4", bins);
  Hist1D::phys(4, "Cross section, 4 < Delta y < 5", bins);
  Hist1D::phys(5, "Cross section, 5 < Delta y < 6", bins);
  Hist1D::phys(101, "Cross section with gap, 1 < Delta y < 2", bins);
  Hist1D::phys(102, "Cross section with gap, 2 < Delta y < 3", bins);
  Hist1D::phys(103, "Cross section with gap, 3 < Delta y < 4", bins);
  Hist1D::phys(104, "Cross section with gap, 3 < Delta y < 5", bins);
  Hist1D::phys(105, "Cross section with gap, 5 < Delta y < 6", bins);


  cout << "UserJets is using " << jetDef.description() << endl;
  cout << "    Gap is between jets, with PTcut = 20 GeV." << endl;
  
  MaxLHEevents = 300000000;
  // The analyzer will turn itself off after this number of calls from deductor.
  max_number_of_events = MaxLHEevents+2;


}


void UserGap::userfunc(const shower_history<hhc>& history)
{
  constexpr double tonb = 389379.338;
  
  for(auto& showered : history)
  {
    /*   showered events   */
    auto& p = showered.state;
    
    double weight = showered.weights.unitary.main*color_weight(showered)*tonb;
    weight *= showered.weights.threshold.main*showered.weights.threshold.reg;
    weight *= showered.weights.threshold.delta;
  
    /* preparing the momenta for FastJet */
    make_pseudojets(theparticles, p);
    
    /*   Clustering the partons using FastJet */
    fj::ClusterSequence clustSeq(theparticles, jetDef);
    
    /*   Define pTcut */
    double pTcut = 20.0;
    /*   Extract inclusive jets (pT > pTcut) */
    incJets0 = clustSeq.inclusive_jets(pTcut);
    incJets = sorted_by_pt(incJets0);
    
    // Find two leading PT jets in allowed rapidity range.
    fj::PseudoJet jet[2];
    unsigned int nj = 0;
    for(unsigned int n = 0; n < incJets.size() && nj < 2; n++)
      if(std::abs(incJets[n].rapidity()) < 4.4) jet[nj++] = incJets[n];
    if(nj < 2) return;

    double Ymin = jet[0].rapidity(), Ymax = jet[1].rapidity();
    double DeltaY = std::abs(Ymax - Ymin);
    if(Ymin > Ymax) std::swap(Ymin, Ymax);
    double pTaverage = 0.5*(jet[0].perp() + jet[1].perp());
    
    double tiny = 1.0e-10;
    double wgap = weight; // wg is weight if nothing in gap, otherwise 0.
    
    for(unsigned int i = 0; i < incJets.size(); i++)
      if(incJets[i].pt() > pTcut &&
         incJets[i].rapidity() < Ymax - tiny &&
         incJets[i].rapidity() > Ymin + tiny )
      {
        wgap = 0.0;
        break;
      }
    
    if(1.0 < DeltaY && DeltaY < 2.0)
    {
      Hist1D::physfill(1, weight, dirac(), pTaverage);
      Hist1D::physfill(101, wgap, dirac(), pTaverage);
    }
    
    if (2.0 < DeltaY && DeltaY < 3.0)
    {
      Hist1D::physfill(2, weight, dirac(), pTaverage);
      Hist1D::physfill(102, wgap, dirac(), pTaverage);
    }
    
    if(3.0 < DeltaY && DeltaY < 4.0)
    {
      Hist1D::physfill(3, weight, dirac(), pTaverage);
      Hist1D::physfill(103, wgap, dirac(), pTaverage);
    }
    
    if (4.0 < DeltaY && DeltaY < 5.0)
    {
      Hist1D::physfill(4, weight, dirac(), pTaverage);
      Hist1D::physfill(104, wgap, dirac(), pTaverage);
    }
    
    if(5.0 < DeltaY && DeltaY < 6.0)
    {
      Hist1D::physfill(5, weight, dirac(), pTaverage);
      Hist1D::physfill(105, wgap, dirac(), pTaverage);
    }
  
  } // Close loop over histories
} // End UserGap::userfunc

